// Solid.h
// Copyright (C) 2012 Miguel Vargas (miguel.vargas@gmail.com)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _Solid_h_
#define _Solid_h_

#include <Basic/Float.h>
#include <Basic/Integer.h>
#include <Basic/Log.h>
#include <Container/Matrix.h>
#include <Container/Vector.h>
#include <FiniteElement/Shape.h>
#include <FiniteElement/ShapeFunctions.h>
#include <FiniteElement/ShapeIntegrationRule.h>
#include <File/File.h>
#include <Math/Formula.h>

#include <string.h>


#define STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE 256
#define STRUCTURE_OPTIMIZATION_USER_FUNCTIONS_COUNT 20
#define STRUCTURE_OPTIMIZATION_USER_FUNCTION_MAX_SIZE 65536

template <typename T>
class Solid
{
	public:

		struct Material
		{
			T poisson_ratio;
			T young_modulus;
			T density;
			T thickness;
			T yield_stress;
		};

		struct Displacement
		{
			int node_id;
			int user_function_x_id;
			T displacement_x;
			bool fixed_x;
			int user_function_y_id;
			T displacement_y;
			bool fixed_y;
			int user_function_z_id;
			T displacement_z;
			bool fixed_z;
		};

		struct NodalForce
		{
			int node_id;
			int user_function_x_id;
			T force_x;
			int user_function_y_id;
			T force_y;
			int user_function_z_id;
			T force_z;
		};

		struct NormalForce
		{
			int element_id;
			int user_function_id;
			T force;
			Vector<int> facet_nodes;
		};

		// General
		bool save_displacement;
		bool save_material;
		bool save_von_mises;
		bool save_sensitivities;
		bool save_filtering;
		bool save_mesh;

		// Gravity
		bool use_mass_forces;
		T gravity;

		// User functions
		T x;
		T y;
		T z;
		T time;
		Vector<Formula<T> > user_function;

		Vector<Material> materials;
		Vector<Displacement> displacements;
		Vector<NodalForce> nodal_forces;
		Vector<NormalForce> normal_forces;
		Vector<IntegrationRule<T> > element_integration_rules;
		Vector<IntegrationRule<T> > facet_integration_rules;

		const ShapeFunctions<T>& shape_functions;
		const Vector<int>& material_index;


		Solid(const char* problem_file_name, const ShapeFunctions<T>& shape_functions, const Vector<int>& material_index) throw(Memory::Exception, File::Exception)
		:	save_displacement(),
			save_material(),
			save_von_mises(),
			save_sensitivities(),
			save_filtering(),
			save_mesh(),
			use_mass_forces(),
			gravity(),
			x(),
			y(),
			z(),
			time(0),
			user_function(STRUCTURE_OPTIMIZATION_USER_FUNCTIONS_COUNT),
			materials(),
			displacements(),
			nodal_forces(),
			normal_forces(),
			element_integration_rules(),
			facet_integration_rules(),
			shape_functions(shape_functions),
			material_index(material_index)
		{
			try
			{
				char section_name[STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE];
				int tmp_integer;

				// Load problem
				File file;
				file.Open(problem_file_name);
				file.SkipComments(';');

				// Load general parameters
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{General}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_displacement = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_material = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_von_mises = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_sensitivities = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_filtering = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(tmp_integer);
				save_mesh = tmp_integer == 1;
				file.SkipComments(';');

				// Load gravity parameters
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{Gravity}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(tmp_integer);
				use_mass_forces = tmp_integer == 1;
				file.SkipComments(';');
				file.Read(gravity);
				file.SkipComments(';');

				// Load user defined functions
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{UserFunctions}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				for (int i = 1; i <= STRUCTURE_OPTIMIZATION_USER_FUNCTIONS_COUNT; ++i)
				{
					char function_text[STRUCTURE_OPTIMIZATION_USER_FUNCTION_MAX_SIZE];
					file.ReadLine(function_text, STRUCTURE_OPTIMIZATION_USER_FUNCTION_MAX_SIZE - 10);
					strncat(function_text, ";x;y;z", 10);
					try
					{
						user_function.entry[i].Define(function_text);
					}
					catch (Memory::Exception&)
					{
						ReThrow();
					}
					catch (Exception&)
					{
						Throw(File::exception_format);
					}
				}

				// Load materials
				int materials_count;
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{Materials}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(materials_count);
				file.SkipComments(';');
				materials.Resize(materials_count);
				for (int m = 1; m <= materials_count; ++m)
				{
					file.Read(materials.entry[m].poisson_ratio);
					file.Read(materials.entry[m].young_modulus);
					file.Read(materials.entry[m].density);
					file.Read(materials.entry[m].thickness);
					file.Read(materials.entry[m].yield_stress);
					file.SkipComments(';');
				}

				// Load displacements conditions
				int displacement_count;
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{Displacement}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(displacement_count);
				file.SkipComments(';');
				displacements.Resize(displacement_count);
				for (int c = 1; c <= displacement_count; ++c)
				{
					displacements.entry[c].fixed_x = false;
					displacements.entry[c].fixed_y = false;
					displacements.entry[c].fixed_z = false;
				}
				for (int c = 1; c <= displacement_count; ++c)
				{
					int fixed_x;
					int fixed_y;
					int fixed_z;

					file.Read(displacements.entry[c].node_id);
					file.Read(displacements.entry[c].user_function_x_id);
					file.Read(displacements.entry[c].displacement_x);
					file.Read(fixed_x);
					displacements.entry[c].fixed_x |= fixed_x == 1;
					file.Read(displacements.entry[c].user_function_y_id);
					file.Read(displacements.entry[c].displacement_y);
					file.Read(fixed_y);
					displacements.entry[c].fixed_y |= fixed_y == 1;
					file.Read(displacements.entry[c].user_function_z_id);
					file.Read(displacements.entry[c].displacement_z);
					file.Read(fixed_z);
					displacements.entry[c].fixed_z |= fixed_z == 1;
					file.SkipComments(';');
				}

				// Load nodal force conditions
				int nodal_force_count;
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{NodalForce}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(nodal_force_count);
				file.SkipComments(';');
				nodal_forces.Resize(nodal_force_count);
				for (int c = 1; c <= nodal_force_count; ++c)
				{
					file.Read(nodal_forces.entry[c].node_id);
					file.Read(nodal_forces.entry[c].user_function_x_id);
					file.Read(nodal_forces.entry[c].force_x);
					file.Read(nodal_forces.entry[c].user_function_y_id);
					file.Read(nodal_forces.entry[c].force_y);
					file.Read(nodal_forces.entry[c].user_function_z_id);
					file.Read(nodal_forces.entry[c].force_z);
					file.SkipComments(';');
				}

				// Load normal force conditions
				int normal_force_count;
				file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
				if (strcmp(section_name, "{NormalForce}") != 0)
				{
					Throw(File::exception_format);
				}
				file.SkipComments(';');
				file.Read(normal_force_count);
				file.SkipComments(';');
				normal_forces.Resize(normal_force_count);
				for (int c = 1; c <= normal_force_count; ++c)
				{
					file.Read(normal_forces.entry[c].element_id);
					file.Read(normal_forces.entry[c].user_function_id);
					file.Read(normal_forces.entry[c].force);
					normal_forces.entry[c].facet_nodes.Resize(shape_functions.mesh.nodes_per_facet);
					for (int f = 1; f <= shape_functions.mesh.nodes_per_facet; ++f)
					{
						file.Read(normal_forces.entry[c].facet_nodes.entry[f]);
					}
					file.SkipComments(';');
				}

				file.Close();

				// Fill integration rules
				int element_integration_points_count;
				int facet_integration_points_count;
				switch (shape_functions.mesh.element_type)
				{
					case shape_triangle:
					{
						element_integration_points_count = (shape_functions.mesh.nodes_per_element == 3) ? 1 : 3;
						facet_integration_points_count = (shape_functions.mesh.nodes_per_facet == 2) ? 1 : 2;
						break;
					}
					case shape_quadrilateral:
					{
						element_integration_points_count = (shape_functions.mesh.nodes_per_element == 4) ? 2 : (shape_functions.mesh.nodes_per_element == 8) ? 2 : 3;
						facet_integration_points_count = (shape_functions.mesh.nodes_per_facet == 2) ? 1 : 2;
						break;
					}
					case shape_tetrahedron:
					{
						element_integration_points_count = (shape_functions.mesh.nodes_per_element == 4) ? 1 : 4;
						facet_integration_points_count = (shape_functions.mesh.nodes_per_facet == 3) ? 1 : 3;
						break;
					}
					case shape_hexahedron:
					{
						element_integration_points_count = (shape_functions.mesh.nodes_per_element == 8) ? 2 : (shape_functions.mesh.nodes_per_element == 20) ? 3 : 3;
						facet_integration_points_count = (shape_functions.mesh.nodes_per_facet == 4) ? 2 : (shape_functions.mesh.nodes_per_facet == 8) ? 2 : 3;
						break;
					}
					default:
					{
						Throw(File::exception_format);
					}
				}
				ShapeIntegrationRule(shape_functions.mesh.element_type, element_integration_points_count, element_integration_rules);
				ShapeIntegrationRule(shape_functions.mesh.facet_type, facet_integration_points_count, facet_integration_rules);

				Log(1, "Problem --------------------------------------------------------------");
				Log(1, "-Save mesh:                %s", save_mesh ? "yes" : "no");
				Log(1, "-Materials used:           %i", materials.size);
				Log(1, "-Displacement conditions:  %i", displacements.size);
				Log(1, "-Nodal force conditions:   %i", nodal_forces.size);
				Log(1, "-Normal force conditions:  %i", normal_forces.size);
				if (use_mass_forces)
				{
					Log(1, "Mass forces:");
					Log(1, "-Gravity applied:          %g", gravity);
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}


		void FillKe(const int element_id, Matrix<T>& Ke) const throw(Memory::Exception)
		{
			// S. W. Sloan
			// A fast stiffness formulation for finite element analysis of two-dimensional solids
			// International Journal for Numerical Methods in Engineering, Vol. 17, pp. 1313-1323, 1981

			try
			{
				int m = shape_functions.mesh.nodes_per_element*shape_functions.nodes.dimension;

				Vector<T> N(shape_functions.mesh.nodes_per_element);
				Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

				int material_id = material_index.entry[element_id];
				Material& material = materials.entry[material_id];

				if (shape_functions.nodes.dimension == 2)
				{
					Matrix<T> Bt(shape_functions.mesh.nodes_per_element*2, 3);
					Matrix<T> DB(3, shape_functions.mesh.nodes_per_element*2);

					T a = material.young_modulus/(1 - material.poisson_ratio*material.poisson_ratio);
					T b = a*material.poisson_ratio;
					T c = a*(1 - material.poisson_ratio)/2;
					T t = material.thickness;

					Ke.Fill(T(0));
					for (int q = 1; q <= element_integration_rules.size; ++q)
					{
						T det_J;
						shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

						for (register int i = 1; i <= shape_functions.mesh.nodes_per_element; ++i)
						{
							register int i2 = i*2;
							register int i1 = i2 - 1;
							T p = dN.entry[i][1];
							T q = dN.entry[i][2];
							Bt.entry[i1][1] = p; Bt.entry[i2][1] = 0;
							Bt.entry[i1][2] = 0; Bt.entry[i2][2] = q;
							Bt.entry[i1][3] = q; Bt.entry[i2][3] = p;
							DB.entry[1][i1] = a*p; DB.entry[1][i2] = b*q;
							DB.entry[2][i1] = b*p; DB.entry[2][i2] = a*q;
							DB.entry[3][i1] = c*q; DB.entry[3][i2] = c*p;
						}

						T weight = element_integration_rules.entry[q].weight;
						for (register int i = 1; i <= m; ++i)
						{
							for (register int j = 1; j <= m; ++j)
							{
								T sum = 0;
								for (register int k = 1; k <= 3; ++k)
								{
									sum += Bt.entry[i][k]*DB.entry[k][j];
								}
								Ke.entry[i][j] += weight*t*sum*det_J;
							}
						}
					}
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					Matrix<T> Bt(shape_functions.mesh.nodes_per_element*3, 6);
					Matrix<T> DB(6, shape_functions.mesh.nodes_per_element*3);

					T z = material.young_modulus/((1 + material.poisson_ratio)*(1 - 2*material.poisson_ratio));
					T a = z*(1 - material.poisson_ratio);
					T b = z*material.poisson_ratio;
					T c = z*(1 - 2*material.poisson_ratio)/2;

					Ke.Fill(T(0));
					for (int q = 1; q <= element_integration_rules.size; ++q)
					{
						T det_J;
						shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

						for (register int i = 1; i <= shape_functions.mesh.nodes_per_element; ++i)
						{
							register int i3 = i*3;
							register int i2 = i3 - 1;
							register int i1 = i3 - 2;
							T p = dN.entry[i][1];
							T q = dN.entry[i][2];
							T r = dN.entry[i][3];
							Bt.entry[i1][1] = p; Bt.entry[i2][1] = 0; Bt.entry[i3][1] = 0;
							Bt.entry[i1][2] = 0; Bt.entry[i2][2] = q; Bt.entry[i3][2] = 0;
							Bt.entry[i1][3] = 0; Bt.entry[i2][3] = 0; Bt.entry[i3][3] = r;
							Bt.entry[i1][4] = q; Bt.entry[i2][4] = p; Bt.entry[i3][4] = 0;
							Bt.entry[i1][5] = 0; Bt.entry[i2][5] = r; Bt.entry[i3][5] = q;
							Bt.entry[i1][6] = r; Bt.entry[i2][6] = 0; Bt.entry[i3][6] = p;
							DB.entry[1][i1] = a*p; DB.entry[1][i2] = b*q; DB.entry[1][i3] = b*r;
							DB.entry[2][i1] = b*p; DB.entry[2][i2] = a*q; DB.entry[2][i3] = b*r;
							DB.entry[3][i1] = b*p; DB.entry[3][i2] = b*q; DB.entry[3][i3] = a*r;
							DB.entry[4][i1] = c*q; DB.entry[4][i2] = c*p; DB.entry[4][i3] = 0;
							DB.entry[5][i1] = 0;   DB.entry[5][i2] = c*r; DB.entry[5][i3] = c*q;
							DB.entry[6][i1] = c*r; DB.entry[6][i2] = 0;   DB.entry[6][i3] = c*p;
						}

						T weight = element_integration_rules.entry[q].weight;
						for (register int i = 1; i <= m; ++i)
						{
							for (register int j = 1; j <= m; ++j)
							{
								T sum = 0;
								for (register int k = 1; k <= 6; ++k)
								{
									sum += Bt.entry[i][k]*DB.entry[k][j];
								}
								Ke.entry[i][j] += weight*sum*det_J;
							}
						}
					}
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}


		void FillU(Vector<T>& u) throw()
		{
			u.Fill(0);

			if (shape_functions.nodes.dimension == 2)
			{
				for (register int i = 1; i <= displacements.size; ++i)
				{
					Displacement& displacement = displacements.entry[i];

					register int i2 = 2*displacement.node_id;
					register int i1 = i2 - 1;
					if (displacement.user_function_x_id == 0)
					{
						u.entry[i1] = displacement.displacement_x;
					}
					else
					{
						x = shape_functions.nodes.coordinate.entry[displacement.node_id][1];
						y = shape_functions.nodes.coordinate.entry[displacement.node_id][2];
						z = 0;
						u.entry[i1] = user_function.entry[displacement.user_function_x_id](x, y, z, time);
					}
					if (displacement.user_function_y_id == 0)
					{
						u.entry[i2] = displacement.displacement_y;
					}
					else
					{
						x = shape_functions.nodes.coordinate.entry[displacement.node_id][1];
						y = shape_functions.nodes.coordinate.entry[displacement.node_id][2];
						z = 0;
						u.entry[i2] = user_function.entry[displacement.user_function_y_id](x, y, z, time);
					}
				}
			}
			else if (shape_functions.nodes.dimension == 3)
			{
				for (register int i = 1; i <= displacements.size; ++i)
				{
					Displacement& displacement = displacements.entry[i];

					register int i3 = 3*displacement.node_id;
					register int i2 = i3 - 1;
					register int i1 = i2 - 1;
					if (displacement.user_function_x_id == 0)
					{
						u.entry[i1] = displacement.displacement_x;
					}
					else
					{
						x = shape_functions.nodes.coordinate.entry[displacement.node_id][1];
						y = shape_functions.nodes.coordinate.entry[displacement.node_id][2];
						z = shape_functions.nodes.coordinate.entry[displacement.node_id][3];
						u.entry[i1] = user_function.entry[displacement.user_function_x_id](x, y, z, time);
					}
					if (displacement.user_function_y_id == 0)
					{
						u.entry[i2] = displacement.displacement_y;
					}
					else
					{
						x = shape_functions.nodes.coordinate.entry[displacement.node_id][1];
						y = shape_functions.nodes.coordinate.entry[displacement.node_id][2];
						z = shape_functions.nodes.coordinate.entry[displacement.node_id][3];
						u.entry[i2] = user_function.entry[displacement.user_function_y_id](x, y, z, time);
					}
					if (displacement.user_function_z_id == 0)
					{
						u.entry[i3] = displacement.displacement_z;
					}
					else
					{
						x = shape_functions.nodes.coordinate.entry[displacement.node_id][1];
						y = shape_functions.nodes.coordinate.entry[displacement.node_id][2];
						z = shape_functions.nodes.coordinate.entry[displacement.node_id][3];
						u.entry[i3] = user_function.entry[displacement.user_function_z_id](x, y, z, time);
					}
				}
			}
		}


		void FillFixed(Vector<bool>& fixed) throw()
		{
			fixed.Fill(false);

			if (shape_functions.nodes.dimension == 2)
			{
				for (register int i = 1; i <= displacements.size; ++i)
				{
					Displacement& displacement = displacements.entry[i];

					register int i2 = 2*displacement.node_id;
					register int i1 = i2 - 1;
					fixed.entry[i1] = displacement.fixed_x;
					fixed.entry[i2] = displacement.fixed_y;
				}
			}
			else if (shape_functions.nodes.dimension == 3)
			{
				for (register int i = 1; i <= displacements.size; ++i)
				{
					Displacement& displacement = displacements.entry[i];

					register int i3 = 3*displacement.node_id;
					register int i2 = i3 - 1;
					register int i1 = i2 - 1;
					fixed.entry[i1] = displacement.fixed_x;
					fixed.entry[i2] = displacement.fixed_y;
					fixed.entry[i3] = displacement.fixed_z;
				}
			}
		}


		void FillF(Vector<T>& f , T* variables ) throw(Memory::Exception)
		{
			try
			{
				f.Fill(0);

				// Fill nodal forces
				if (shape_functions.nodes.dimension == 2)
				{
					for (register int i = 1; i <= nodal_forces.size; ++i)
					{
						NodalForce& node_force = nodal_forces.entry[i];

						register int i2 = 2*node_force.node_id;
						register int i1 = i2 - 1;
						if (node_force.user_function_x_id == 0)
						{
							f.entry[i1] = node_force.force_x;
						}
						else
						{
							x = shape_functions.nodes.coordinate.entry[node_force.node_id][1];
							y = shape_functions.nodes.coordinate.entry[node_force.node_id][2];
							z = 0;
							f.entry[i1] = user_function.entry[node_force.user_function_x_id](x, y, z, time);
						}
						if (node_force.user_function_y_id == 0)
						{
							f.entry[i2] = node_force.force_y;
						}
						else
						{
							x = shape_functions.nodes.coordinate.entry[node_force.node_id][1];
							y = shape_functions.nodes.coordinate.entry[node_force.node_id][2];
							z = 0;
							f.entry[i2] = user_function.entry[node_force.user_function_y_id](x, y, z, time);
						}
					}
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					for (register int i = 1; i <= nodal_forces.size; ++i)
					{
						NodalForce& node_force = nodal_forces.entry[i];

						register int i3 = 3*node_force.node_id;
						register int i2 = i3 - 1;
						register int i1 = i2 - 1;
						if (node_force.user_function_x_id == 0)
						{
							f.entry[i1] = node_force.force_x;
						}
						else
						{
							x = shape_functions.nodes.coordinate.entry[node_force.node_id][1];
							y = shape_functions.nodes.coordinate.entry[node_force.node_id][2];
							z = shape_functions.nodes.coordinate.entry[node_force.node_id][3];
							f.entry[i1] = user_function.entry[node_force.user_function_x_id](x, y, z, time);
						}
						if (node_force.user_function_y_id == 0)
						{
							f.entry[i2] = node_force.force_y;
						}
						else
						{
							x = shape_functions.nodes.coordinate.entry[node_force.node_id][1];
							y = shape_functions.nodes.coordinate.entry[node_force.node_id][2];
							z = shape_functions.nodes.coordinate.entry[node_force.node_id][3];
							f.entry[i2] = user_function.entry[node_force.user_function_y_id](x, y, z, time);
						}
						if (node_force.user_function_z_id == 0)
						{
							f.entry[i3] = node_force.force_z;
						}
						else
						{
							x = shape_functions.nodes.coordinate.entry[node_force.node_id][1];
							y = shape_functions.nodes.coordinate.entry[node_force.node_id][2];
							z = shape_functions.nodes.coordinate.entry[node_force.node_id][3];
							f.entry[i3] = user_function.entry[node_force.user_function_z_id](x, y, z, time);
						}
					}
				}

				// Fill normal forces
				Vector<T> facet_N(shape_functions.mesh.nodes_per_facet);

				if (shape_functions.nodes.dimension == 2)
				{
					for (register int i = 1; i <= normal_forces.size; ++i)
					{
						NormalForce& facet_force = normal_forces.entry[i];
						int material_id = material_index.entry[facet_force.element_id];
						Material& material = materials.entry[material_id];

						for (int q = 1; q <= facet_integration_rules.size; ++q)
						{
							// Normal vector components
							int n1 = facet_force.facet_nodes.entry[1];
							int n2 = facet_force.facet_nodes.entry[2];
							T nfx = -(shape_functions.nodes.coordinate.entry[n2][2] - shape_functions.nodes.coordinate.entry[n1][2]);
							T nfy = shape_functions.nodes.coordinate.entry[n2][1] - shape_functions.nodes.coordinate.entry[n1][1];
							T sn = sqrt(nfx*nfx + nfy*nfy);
							nfx /= sn;
							nfy /= sn;

							T det_J;
							shape_functions.FacetShapeFunctions(facet_force.facet_nodes, facet_integration_rules.entry[q].point, facet_N, det_J);

							T w = facet_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_facet; ++j)
							{
								register int n = (facet_force.facet_nodes.entry[j] - 1)*2;
								if (facet_force.user_function_id == 0)
								{
									f.entry[n + 1] += w*material.thickness*(facet_force.force*nfx)*facet_N.entry[j]*det_J;
									f.entry[n + 2] += w*material.thickness*(facet_force.force*nfy)*facet_N.entry[j]*det_J;
								}
								else
								{
									x = shape_functions.nodes.coordinate.entry[facet_force.facet_nodes.entry[j]][1];
									y = shape_functions.nodes.coordinate.entry[facet_force.facet_nodes.entry[j]][2];
									z = 0;
									T force = user_function.entry[facet_force.user_function_id](x, y, z, time);
									f.entry[n + 1] += w*material.thickness*(force*nfx)*facet_N.entry[j]*det_J;
									f.entry[n + 2] += w*material.thickness*(force*nfy)*facet_N.entry[j]*det_J;
								}
							}
						}
					}
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					for (register int i = 1; i <= normal_forces.size; ++i)
					{
						NormalForce& facet_force = normal_forces.entry[i];

						for (int q = 1; q <= facet_integration_rules.size; ++q)
						{
							// Normal vector components
							int n1 = facet_force.facet_nodes.entry[1];
							int n2 = facet_force.facet_nodes.entry[2];
							int n3 = facet_force.facet_nodes.entry[3];
							T ax = shape_functions.nodes.coordinate.entry[n2][1] - shape_functions.nodes.coordinate.entry[n1][1];
							T ay = shape_functions.nodes.coordinate.entry[n2][2] - shape_functions.nodes.coordinate.entry[n1][2];
							T az = shape_functions.nodes.coordinate.entry[n2][3] - shape_functions.nodes.coordinate.entry[n1][3];
							T bx = shape_functions.nodes.coordinate.entry[n3][1] - shape_functions.nodes.coordinate.entry[n1][1];
							T by = shape_functions.nodes.coordinate.entry[n3][2] - shape_functions.nodes.coordinate.entry[n1][2];
							T bz = shape_functions.nodes.coordinate.entry[n3][3] - shape_functions.nodes.coordinate.entry[n1][3];
							T nfx = ay*bz - az*by;
							T nfy = az*bx - ax*bz;
							T nfz = ax*by - ay*bx;
							T sn = sqrt(nfx*nfx + nfy*nfy + nfz*nfz);
							nfx /= sn;
							nfy /= sn;
							nfz /= sn;

							T det_J;
							shape_functions.FacetShapeFunctions(facet_force.facet_nodes, facet_integration_rules.entry[q].point, facet_N, det_J);

							T w = facet_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_facet; ++j)
							{
								register int n = (facet_force.facet_nodes.entry[j] - 1)*3;
								if (facet_force.user_function_id == 0)
								{
									f.entry[n + 1] += w*(facet_force.force*nfx)*facet_N.entry[j]*det_J;
									f.entry[n + 2] += w*(facet_force.force*nfy)*facet_N.entry[j]*det_J;
									f.entry[n + 3] += w*(facet_force.force*nfz)*facet_N.entry[j]*det_J;
								}
								else
								{
									x = shape_functions.nodes.coordinate.entry[facet_force.facet_nodes.entry[j]][1];
									y = shape_functions.nodes.coordinate.entry[facet_force.facet_nodes.entry[j]][2];
									z = shape_functions.nodes.coordinate.entry[facet_force.facet_nodes.entry[j]][3];
									T force = user_function.entry[facet_force.user_function_id](x, y, z, time);
									f.entry[n + 1] += w*(force*nfx)*facet_N.entry[j]*det_J;
									f.entry[n + 2] += w*(force*nfy)*facet_N.entry[j]*det_J;
									f.entry[n + 3] += w*(force*nfz)*facet_N.entry[j]*det_J;
								}
							}
						}
					}
				}

				// Mass forces
				if (use_mass_forces)
				{
					Vector<T> N(shape_functions.mesh.nodes_per_element);
					Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

					const Matrix<int>& connectivity = shape_functions.mesh.connectivity;

					if (shape_functions.nodes.dimension == 2)
					{
						for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
						{
							double element_weight=(double)variables[element_id-1];
							int material_id = material_index.entry[element_id];
							Material& material = materials.entry[material_id];

							for (int q = 1; q <= element_integration_rules.size; ++q)
							{
								T det_J;
								shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

								T w = element_integration_rules.entry[q].weight;
								for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
								{
									register int n = connectivity.entry[element_id][j]*2; // y axis
									f.entry[n] -= w*material.thickness*material.density*gravity*N.entry[j]*det_J*element_weight;
								}
							}
						}
					}
					else if (shape_functions.nodes.dimension == 3)
					{
						for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
						{
							double element_weight=(double)variables[element_id-1];
							int material_id = material_index.entry[element_id];
							Material& material = materials.entry[material_id];

							for (int q = 1; q <= element_integration_rules.size; ++q)
							{
								T det_J;
								shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

								T w = element_integration_rules.entry[q].weight;
								for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
								{
									register int n = connectivity.entry[element_id][j]*3; // z axis
									f.entry[n] -= w*material.density*gravity*N.entry[j]*det_J*element_weight;
								}
							}
						}
					}
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}


		double TotalWeight() throw(Memory::Exception)
		{
			try
			{
				Vector<T> N(shape_functions.mesh.nodes_per_element);
				Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

				double total_weight = 0;
				if (shape_functions.nodes.dimension == 2)
				{
					for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
					{
						int material_id = material_index.entry[element_id];
						Material& material = materials.entry[material_id];

						for (int q = 1; q <= element_integration_rules.size; ++q)
						{
							T det_J;
							shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

							T w = element_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
							{
								total_weight += w*material.thickness*material.density*gravity*N.entry[j]*det_J;
							}
						}
					}
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
					{
						int material_id = material_index.entry[element_id];
						Material& material = materials.entry[material_id];

						for (int q = 1; q <= element_integration_rules.size; ++q)
						{
							T det_J;
							shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

							T w = element_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
							{
								total_weight += w*material.density*gravity*N.entry[j]*det_J;
							}
						}
					}
				}

				return total_weight;
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}

		double GetElementSize(int element_id) throw(Memory::Exception)
		{
			try
			{
				Vector<T> N(shape_functions.mesh.nodes_per_element);
				Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

				if (shape_functions.nodes.dimension == 2)
				{
					double area = 0;
					for (int q = 1; q <= element_integration_rules.size; ++q)
					{
						T det_J;
						shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

						T w = element_integration_rules.entry[q].weight;
						for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
						{
							area += w*N.entry[j]*det_J;
						}
					}
					return area;
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					double volume = 0;
					for (int q = 1; q <= element_integration_rules.size; ++q)
					{
						T det_J;
						shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

						T w = element_integration_rules.entry[q].weight;
						for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
						{
							volume += w*N.entry[j]*det_J;
						}
					}
					return volume;
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
			return 0;
		}


		double GetTotalSize() throw(Memory::Exception)
		{
			try
			{
				Vector<T> N(shape_functions.mesh.nodes_per_element);
				Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

				if (shape_functions.nodes.dimension == 2)
				{
					double area = 0;
					for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
					{
						for (int q = 1; q <= element_integration_rules.size; ++q)
						{
							T det_J;
							shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

							T w = element_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
							{
								area += w*N.entry[j]*det_J;
							}
						}
					}
					return area;
				}
				else if (shape_functions.nodes.dimension == 3)
				{
					double volume = 0;
					for (int element_id = 1; element_id <= shape_functions.mesh.elements_count; ++element_id)
					{
						for (int q = 1; q <= element_integration_rules.size; ++q)
						{
							T det_J;
							shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

							T w = element_integration_rules.entry[q].weight;
							for (register int j = 1; j <= shape_functions.mesh.nodes_per_element; ++j)
							{
								volume += w*N.entry[j]*det_J;
							}
						}
					}
					return volume;
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}

		void GetStrain(const int element_id, const Vector<T>& u, Vector<Vector<T> >& strain) throw(Memory::Exception)
		{
			Assert(strain.size == element_integration_rules.size);
			Assert(strain.entry[1].size == (shape_functions.nodes.dimension - 1)*3);

			try
			{
				Vector<T> N(shape_functions.mesh.nodes_per_element);
				Matrix<T> dN(shape_functions.mesh.nodes_per_element, shape_functions.nodes.dimension);

				Matrix<T> du_dx(shape_functions.nodes.dimension, shape_functions.nodes.dimension);

				for (int q = 1; q <= element_integration_rules.size; ++q)
				{
					T det_J;
					shape_functions.ElementShapeFunctions(element_id, element_integration_rules.entry[q].point, N, dN, det_J);

					du_dx.Fill((T)0);
					for (int du = 1; du <= shape_functions.nodes.dimension; ++du)
					{
						for (register int dx = 1; dx <= shape_functions.nodes.dimension; ++dx)
						{
							for (register int i = 1; i <= shape_functions.mesh.nodes_per_element; ++i)
							{
								register int k = (shape_functions.mesh.connectivity.entry[element_id][i] - 1)*shape_functions.nodes.dimension + du;
								du_dx.entry[du][dx] += dN.entry[i][dx]*u.entry[k];
							}
						}
					}
					if (shape_functions.nodes.dimension == 2)
					{
						strain.entry[q].entry[1] = du_dx.entry[1][1];
						strain.entry[q].entry[2] = du_dx.entry[2][2];
						strain.entry[q].entry[3] = du_dx.entry[1][2] + du_dx.entry[2][1];
					}
					else if (shape_functions.nodes.dimension == 3)
					{
						strain.entry[q].entry[1] = du_dx.entry[1][1];
						strain.entry[q].entry[2] = du_dx.entry[2][2];
						strain.entry[q].entry[3] = du_dx.entry[3][3];
						strain.entry[q].entry[4] = du_dx.entry[1][2] + du_dx.entry[2][1];
						strain.entry[q].entry[5] = du_dx.entry[2][3] + du_dx.entry[3][2];
						strain.entry[q].entry[6] = du_dx.entry[3][1] + du_dx.entry[1][3];
					}
				}
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}


		void GetStress(const int element_id, const Vector<Vector<T> >& strain, Vector<Vector<T> >& stress) throw()
		{
			Assert(stress.size == element_integration_rules.size);
			Assert(stress.entry[1].size == (shape_functions.nodes.dimension - 1)*3);

			int material_id = material_index.entry[element_id];
			Material& material = materials.entry[material_id];
			T E = material.young_modulus;
			T v = material.poisson_ratio;

			if (shape_functions.nodes.dimension == 2)
			{
				T a = E/(1 - v*v);
				T b = (1 - v)/2;

				for (int q = 1; q <= element_integration_rules.size; ++q)
				{
					stress.entry[q].entry[1] = a*strain.entry[q].entry[2]*v + a*strain.entry[q].entry[1];
					stress.entry[q].entry[2] = a*strain.entry[q].entry[1]*v + a*strain.entry[q].entry[2];
					stress.entry[q].entry[3] = a*b*strain.entry[q].entry[3];
				}
			}
			else if (shape_functions.nodes.dimension == 3)
			{
				T a = E*(1 - v)/((1 + v)*(1 - 2*v));
				T b = v/(1 - v);
				T c = (1 - 2*v)/(2 - 2*v);

				for (int q = 1; q <= element_integration_rules.size; ++q)
				{
					stress.entry[q].entry[1] = a*b*strain.entry[q].entry[3] + a*b*strain.entry[q].entry[2] + a*strain.entry[q].entry[1];
					stress.entry[q].entry[2] = a*b*strain.entry[q].entry[3] + a*strain.entry[q].entry[2] + a*b*strain.entry[q].entry[1];
					stress.entry[q].entry[3] = a*strain.entry[q].entry[3] + a*b*strain.entry[q].entry[2] + a*b*strain.entry[q].entry[1];
					stress.entry[q].entry[4] = a*c*strain.entry[q].entry[4];
					stress.entry[q].entry[5] = a*c*strain.entry[q].entry[5];
					stress.entry[q].entry[6] = a*c*strain.entry[q].entry[6];
				}
			}
		}


		void GetVonMises(const Vector<Vector<T> >& stress, Vector<T>& von_mises) throw()
		{
			Assert(von_mises.size == element_integration_rules.size);

			if (shape_functions.nodes.dimension == 2)
			{
				for (int q = 1; q <= element_integration_rules.size; ++q)
				{
					// J. E. Akin. Finite Element Analysis with Error Estimators. An Introduction to the FEM and Adaptive Error Analysis for Engineering Students. Elsevier. 2005. pp. 404
					T sx = stress.entry[q].entry[1];
					T sy = stress.entry[q].entry[2];
					T txy = stress.entry[q].entry[3];
					T J2 = ((sx - sy)*(sx - sy) + (sx + sy)*(sx + sy) + 6*txy*txy)/6;
					von_mises.entry[q] = sqrt(3*J2);
				}
			}
			else if (shape_functions.nodes.dimension == 3)
			{
				for (int q = 1; q <= element_integration_rules.size; ++q)
				{
					// E. Oate. Structural Analysis with the Finite Element Method. Linear Statics. Volume 1. Basis and Solids. Springer. 2009. pp. 255-256
					T sx = stress.entry[q].entry[1];
					T sy = stress.entry[q].entry[2];
					T sz = stress.entry[q].entry[3];
					T txy = stress.entry[q].entry[4];
					T tyz = stress.entry[q].entry[5];
					T txz = stress.entry[q].entry[6];
					T J2 = ((sx - sy)*(sx - sy) + (sx - sz)*(sx - sz) + (sy - sz)*(sy - sz) + 6*(txy*txy + tyz*tyz + txz*txz))/6;
					von_mises.entry[q] = sqrt(3*J2);
				}
			}
		}


		void WriteMaterialGaussPoints(File& file) throw(Memory::Exception, File::Exception)
		{
			static const char* element_name[] =
			{
				"(Undefined)",
				"Linear",
				"Triangle",
				"Quadrilateral",
				"Tetrahedra",
				"Hexahedra"
			};

			try
			{
				file.Write("GaussPoints \"MaterialGaussPoints\" ElemType ");
				file.Write(element_name[shape_functions.mesh.element_type]);
				file.Write("\n");
				file.Write("Number Of Gauss Points: 1\n");
				file.Write("Natural Coordinates: internal\n");
				file.Write("End GaussPoints\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}

	
		void WriteMaterial(File& file, const Vector<int>& element_index,const Vector<double>& design_variables, int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			  	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"Material\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Scalar OnGaussPoints \"MaterialGaussPoints\"\n");
				file.Write("Values\n");

				for (int e = 1; e <= element_index.size; ++e)
				{
					int element_id = element_index.entry[e];
					double material_proportion = design_variables.entry[ element_id ];
					file.Write(element_id, format_integer);
					file.Write(" ");				
					if(material_proportion > 0.5)	material_proportion = 1.0;
					else				material_proportion = 0.0;
					file.Write(material_proportion, format_float);
					file.Write("\n");
				}

				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}
		
		void WriteDisplacements(File& file, const Vector<int>& node_index, const Vector<bool>& material_nodes , const Vector<T>& u, const Vector<double>& minimal_displacements_per_dimension , int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			  	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"Displacement\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Vector OnNodes\n");
				file.Write("Values\n");
				for (int n = 1, i = 1; n <= node_index.size; ++n, i += 2)
				{
					if( material_nodes.entry[ n ] == true )
					{
					      file.Write(node_index.entry[n], format_integer);
					      file.Write(" ");
					      file.Write(u.entry[i], format_float);
					      file.Write(" ");
					      file.Write(u.entry[i + 1], format_float);
					      file.Write(" ");
					      file.Write((T)0, format_float);
					      file.Write(" ");
					      T displacement = sqrt(u.entry[i]*u.entry[i] + u.entry[i + 1]*u.entry[i + 1]);
					      file.Write(displacement, format_float);
					      file.Write("\n");
					}
					else
					{
					      file.Write(node_index.entry[n], format_integer);
					      file.Write(" ");
					      file.Write( minimal_displacements_per_dimension.entry[ 1 ] , format_float);
					      file.Write(" ");
					      file.Write( minimal_displacements_per_dimension.entry[ 2 ] , format_float);
					      file.Write(" ");
					      file.Write((T)0, format_float);
					      file.Write(" ");
					      file.Write((T)0, format_float);
					      file.Write("\n");
					}
				}
				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}

		void WriteStress(File& file, const Vector<int>& element_index,const Vector<double>& design_variables , const Vector<Vector<double> >& vmelem_coord ,  int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			  	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"Stress\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Vector OnGaussPoints \"MaterialGaussPoints\"\n");
				file.Write("Values\n");

				for (int e = 1; e <= element_index.size; ++e)
				{
					int element_id = element_index.entry[e];
					file.Write(element_id, format_integer);
					file.Write(" ");
					if(  design_variables.entry[ element_id ] > 0.5)
					{
					    file.Write(vmelem_coord.entry[ element_id ].entry[ 1 ], format_float );
					    file.Write(" ");
					    file.Write(vmelem_coord.entry[ element_id ].entry[ 2 ], format_float );
					    file.Write(" ");
					    file.Write(vmelem_coord.entry[ element_id ].entry[ 3 ], format_float );
					    file.Write("\n");
					}
					else
					{
					    file.Write("0.00000e-0 0.00000e-0 0.00000e-0\n");
					}
				}

				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}
		
		void WriteVonMises(File& file, const Vector<int>& element_index,const Vector<double>& design_variables , const Vector<double>& vme ,  int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			  	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"VonMises\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Scalar OnGaussPoints \"MaterialGaussPoints\"\n");
				file.Write("Values\n");

				for ( int e = 1; e <= element_index.size; ++e )
				{
					int element_id = element_index.entry[e];
					double material_proportion = design_variables.entry[element_id];
					file.Write(element_id, format_integer);
					file.Write(" ");				
					if(material_proportion > 0.5)	material_proportion = vme.entry[ element_id ];
					else				material_proportion = 0.0;
					file.Write(material_proportion, format_float);
					file.Write("\n");
				}

				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}
		
		void WriteSensitivities(File& file, const Vector<int>& element_index,const Vector<double>& sensitivities, int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			  	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"Sensitivities\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Scalar OnGaussPoints \"MaterialGaussPoints\"\n");
				file.Write("Values\n");

				for (int e = 1; e <= element_index.size; ++e)
				{
					int element_id = element_index.entry[e];
					double sensitivity = sensitivities.entry[element_id];
					file.Write(element_id, format_integer);
					file.Write(" ");							
					file.Write(sensitivity, format_float);
					file.Write("\n");
				}

				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}

		void WriteFiltering(File& file, const Vector<int>& element_index,const Vector<double>& filtering, int iteration) throw(Memory::Exception, File::Exception)
		{
			try
			{
			    	FormatFloat format_float(false, false, 1, 5, FormatFloat::exponential);
				FormatInteger format_integer(false, false, 1, FormatInteger::decimal);

				file.Write("Result \"Filtering\" \"StructureOptimization\" ");
				file.Write(iteration, format_integer);
				file.Write(" Scalar OnGaussPoints \"MaterialGaussPoints\"\n");
				file.Write("Values\n");

				for (int e = 1; e <= element_index.size; ++e)
				{
					int element_id = element_index.entry[ e ];
					double filter = filtering.entry[ element_id ];
					file.Write(element_id, format_integer);
					file.Write(" ");				
					file.Write(filter, format_float);
					file.Write("\n");
				}

				file.Write("End Values\n\n");
			}
			catch (Exception&)
			{
				ReThrow();
			}
		}


	private:

		Solid& operator = (const Solid&) throw()
		{
			return *this;
		}
};

#endif

