
#ifndef _SIMP_h
#define _SIMP_h_

#include "Solid.h"

#include <Basic/Log.h>
#include <Basic/Macros.h>
#include <Basic/Debug.h>
#include <Basic/Assert.h>
#include <Basic/Memory.h>
#include <Container/CSRMatrix.h>
#include <Container/Matrix.h>
#include <Container/Vector.h>
#include <FiniteElement/Assembler.h>
#include <FiniteElement/Geometry.h>
#include <FiniteElement/ShapeFunctions.h>
#include <Math/Distribution.h>
#include <Solver/SolverParameters.h>

#define PREFIX_MAXIMUM_LENGTH 900
#define PATH_MAXIMUM_LENGTH 1000

template <typename T>
class SIMP
{
    public:
		// Method parameters
		int 	optimization_method;
		int 	maximal_iterations;
		int 	maximal_global_iterations;
		double  thickness_empty_material;
		double  volume_constraint;
		double  sf_constraint;	
		double  eps_convergence;
		int  	variance_convergence_parameter;
		double  YS;
		int 	p;
		double  ratio;
		double  mov;
		
		// FEM data
		const int 		M , N;				//Global and elemental matrix size
		Vector<Matrix<double> > Ke;
		Vector<Vector<double> > ue;
		Vector<double> 		elemental_vonmises;
		Vector<Vector<double> > elemental_vonmises_per_dimension;
		Vector<double> 		elemental_volumes;
		Vector<double> 		u;
		Vector<bool> 		fixed;
		bool 			valid_solution;
		double 			total_weight;
		
		// Iterative data
		Vector<double> 	design_variables;
		int 		iteration;
		int		global_iteration;
		double 		compliance;
		double 		volume;
		double		security_factor;
		double		objective_function;
		double 		lagrange;
		Vector<double> 	sensitivities;
		Vector<double> 	filtering;
		Vector<double> 	next_design_variables;
		
		// Initial data
		double domain_volume;
		time_t start_time;

		//Stopping criterion data
		double 		design_variables_convergence;
		double 		security_factor_convergence;
		double 		historical_variance_convergence;
		Vector<double>  historical_variance;
		double		volume_convergence;
		double 		maximal_filtering;
		double 		maximal_vonmises;
		double 		last_maximal_vonmises;
		double 		last_volume_constraint;
				
		// Displaements and stress
		Vector<double> 	displacement_avg_coord;
		double 		displacement_avg;
		double 		stress_avg;
		double 		stress_sd;
		Vector<bool> 	material_nodes;
		Vector<double> 	minimal_displacements_per_dimension;
		
		//Neighborhoods
		Matrix<double> 		  elemental_centroids;
		Vector < Matrix<double> >  elemental_neighborhoods;

		
		SIMP( const char* optimization_file_name,
				       Geometry<double>& geometry,
				       Vector<int>& node_index,
				       Vector<int>& element_index,
				       Solid<double>& solid,
				       Assembler assembler
				      ) throw(Memory::Exception)
		:	
			YS(solid.materials.entry[1].yield_stress),
			
			M( node_index.size * geometry.nodes.dimension ),			
			N( geometry.mesh.nodes_per_element*geometry.nodes.dimension ),	
 			Ke( element_index.size ),
			ue( element_index.size ),
			elemental_vonmises( element_index.size ),
			elemental_vonmises_per_dimension( element_index.size ),
			elemental_volumes( element_index.size ),
			u( M ),
			fixed( M ),
			
			design_variables( element_index.size ),
			iteration( 1 ),
			global_iteration( 1 ),
			sensitivities(element_index.size),
			filtering(element_index.size),
			next_design_variables(element_index.size),
			
			start_time(time(NULL)),
			
			displacement_avg_coord(geometry.nodes.dimension*2),
			material_nodes(node_index.size),
			minimal_displacements_per_dimension(geometry.nodes.dimension),
			
			elemental_centroids( element_index.size , geometry.nodes.dimension ),
			elemental_neighborhoods( element_index.size )		
		{
		
		    //Reading parameters
			char section_name[STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE];
			// Load problem
			File file;
			file.Open(optimization_file_name);
			file.SkipComments(';');
			
			// Load general parameters
			file.Read(section_name, STRUCTURE_OPTIMIZATION_SECTION_NAME_MAX_SIZE);
			if (strcmp(section_name, "{Optimization}") != 0)
			{
				Throw(File::exception_format);
			}
			file.SkipComments(';');
			file.Read(maximal_iterations);
			file.SkipComments(';');
			file.Read(thickness_empty_material);
			file.SkipComments(';');
			file.Read(p);
			file.SkipComments(';');
			file.Read(ratio);
			file.SkipComments(';');
			file.Read(mov);
			file.SkipComments(';');
			file.Read(optimization_method);
			file.SkipComments(';');
			file.Read(volume_constraint);
			file.SkipComments(';');
			file.Read(sf_constraint);
			file.SkipComments(';');
			file.Read(eps_convergence);
			file.SkipComments(';');
			file.Read(variance_convergence_parameter);
			file.SkipComments(';');
			file.Read(maximal_global_iterations);
			file.SkipComments(';');
			
			historical_variance.Resize(variance_convergence_parameter);
		
		    // Initializing components
			for( int i = 1 ; i <= element_index.size ; ++i )
			{
				int element_id = element_index.entry[ i ];
				design_variables.entry[ element_id ] = 1.0;		
				//Elemental foces and efforts vectors
				ue.entry[ element_id ].Resize( N );
				//Elemental stiffness matrices (also fill them)
				Ke.entry[ element_id ].Resize( N , N );
				solid.FillKe( element_id , Ke.entry[ element_id ] );
				// Elemental elemental_volumes
				elemental_volumes.entry[ element_id ] = solid.GetElementSize( element_id );
				// Memory for stress and displacements vectors
				int R = ( geometry.nodes.dimension - 1 ) * 3;
				elemental_vonmises_per_dimension.entry[ element_id ].Resize( R );
			}
			// Assemble initial displacement vector
 			Vector<bool> global_fixed(M);
			Vector<double> global_u(M);
 			solid.FillU(global_u);
 			solid.FillFixed(global_fixed);
 			assembler.AssembleV(global_u, u);
 			assembler.AssembleV(global_fixed, fixed);
			// Calculate centroids position of each element
			get_elemental_centroids( element_index ,geometry );
			// Get neighborhood for each element centroid
			get_elemental_neighborhood( element_index , geometry );
		}
		
		
		void FEM(Vector<int>& element_index,
				Vector<int>& node_index,
				SolverParameters<double>& solver_parameters,
				Solid<double>& solid,
				ShapeFunctions<double>& shape_functions,
				Geometry<double>& geometry,
				Assembler assembler
 			      ) throw()
		{
		      	// Assemble global matrices and vectors
			    //Global stiffness matrix
			CSRMatrix<double> K(M, M);
			assembler.AllocateMatrix(K);
			K.Fill(0);
			Matrix<double> KeTMP(N, N);
			for (int i = 1; i <= element_index.size; ++i)
			{
				int element_id = element_index.entry[i];
				KeTMP = Ke.entry[i];
				KeTMP = KeTMP * pow( design_variables.entry[i] ,(unsigned int)p );	//Weight of design variable
				assembler.AssembleAe(element_id, KeTMP, K);
			}
			  // Assemble forces vector
			Vector<double> f(M);
			{
				Vector<double> global_f(M);
				solid.FillF(global_f,design_variables.entry);
				assembler.AssembleV(global_f, f);
			}

			// Initialize solver
			Solver<double>* solver = solver_parameters.Instanciate(K, u, f, fixed, false);
			// Solve system of equations
			solver->CompensateFixed();
			valid_solution = solver->Calculate();

			// Un-reorder values in nodes
			Vector<double> global_u(M);
			for (int n = 1; n <= node_index.size; ++n)
			{
				int global_n = node_index.entry[n];
				for (int d = 1; d <= geometry.nodes.dimension; ++d)
				{
					global_u.entry[(global_n - 1) * geometry.nodes.dimension + d] =
					u.entry[ (n - 1) * geometry.nodes.dimension + d ] ;
				}
			}

			// Calculate stress per element
			maximal_vonmises = 0.0;
			int Q = solid.element_integration_rules.size;
			int R = (shape_functions.nodes.dimension - 1) * 3;
			Vector<Vector<double> > strain(Q);
			Vector<Vector<double> > stress(Q);
			Vector<double> von_mises(Q);
			for (int q = 1; q <= Q; ++q)
			{
				strain.entry[q].Resize(R);
				stress.entry[q].Resize(R);
			}
			for (int e = 1; e <= element_index.size; ++e)
			{
				int element_id = element_index.entry[e];
				if( design_variables.entry[element_id] < 0.5 )
				{
				      elemental_vonmises.entry[ element_id ] = 1e-100;
				}
				else
				{
				      solid.GetStrain(element_id, global_u, strain);
				      solid.GetStress(element_id, strain, stress);
				      solid.GetVonMises(stress, von_mises);
				      
				      // Get average stress per axis
				      Vector<double> avg( R );
				      avg.Fill(0);
				      for (int i = 1; i <= Q; ++i )
				      {
					  for (int j = 1; j <= R; ++j )
					  {
					      avg.entry[ j ] += stress.entry[ i ].entry[ j ];
					  }
				      }
				      for (int j = 1; j <= R; ++j )
				      {
					  elemental_vonmises_per_dimension.entry[element_id].entry[ j ] = fabs(avg.entry[ j ] / Q );
				      }
				      // Get elemental stress
				      double vmStress = 0.0;
				      for (int i = 1; i <= Q; ++i)
				      {
					      vmStress += von_mises.entry[ i ];
				      }
				      vmStress /= (double)Q;
				      elemental_vonmises.entry[ element_id ] = vmStress;
				      // Get max elemental stress
				      if( vmStress > maximal_vonmises )
				      {
					  maximal_vonmises = vmStress;
				      }
				}
			}
			// Get security security_factor
			security_factor = maximal_vonmises/YS;
			// Calculate displacements per element
			int nodes_per_element = geometry.mesh.nodes_per_element;
			int dimension = geometry.nodes.dimension;
			for (int i = 1; i <= node_index.size; ++i )
			{
			    material_nodes.entry[ i ] = false;
			}
			for (int i = 1; i <= dimension; ++i )
			{
			    minimal_displacements_per_dimension.entry[ i ] = 1e100;
			}
			for (int i = 1; i <= element_index.size; ++i)
			{
			    int element_id = element_index.entry[ i ];
			    // Get average displacement per axis
			    for (int j = 1; j <= nodes_per_element ; ++j )
			    {
				for (int k = 1; k <= dimension ; ++k)
				{  
				    int node = geometry.mesh.connectivity.entry[ element_id ][ j ];
				    double displacement_node_dimension = u.entry[ ( ( node - 1 ) * dimension ) + k ];
				    ue.entry[ element_id ].entry[(j - 1) * dimension + (k)] = displacement_node_dimension ;
				    
				    if( design_variables.entry[ element_id ] >= 0.5 )
				    {
					material_nodes.entry[ node ] = true;
					if( displacement_node_dimension < minimal_displacements_per_dimension.entry [ k ] )
					{
					    minimal_displacements_per_dimension.entry [ k ] = displacement_node_dimension;
					}
				    }
				}
			    }
			}
			
			
		
			// Calculate total weight
			total_weight = solid.TotalWeight();

			// Free solver resources
			delete solver;
		      
		}
		
		void get_elemental_centroids( Vector<int>& element_index,
					      Geometry<double>& geometry )
		{
		    int dimension = geometry.nodes.dimension;
		    int nodes_per_element = geometry.mesh.nodes_per_element;
		    domain_volume = 0.0;
		    //Get elements centroids
		    for( int i = 1 ; i <= element_index.size ; ++i )
		    {
			int element_id = element_index.entry[ i ];
			domain_volume += elemental_volumes.entry[ element_id ];
			
			Vector<double> centroid(dimension);
			for ( int dim = 1 ; dim <= dimension ; dim++ )
			{
			    centroid.entry[ dim ] = 0.0;
			}
			// Get sum of nodes per dimension
			for ( int j = 1 ; j <= nodes_per_element ; j++ )
			{
			    int node_id = geometry.mesh.connectivity.entry[ element_id ][ j ];
			    for ( int dim = 1 ; dim <= dimension ; dim++ )
			    {
				centroid.entry[ dim ] += geometry.nodes.coordinate.entry[ node_id ][ dim ];
			    }
			}
			// Get centroid 
			for ( int dim = 1 ; dim <= dimension ; dim++ )
			{
			    elemental_centroids.entry[ element_id ][ dim ] = centroid.entry[ dim ] / nodes_per_element;
			}
		    }
		}
		
		void get_elemental_neighborhood( Vector<int>& element_index,
						 Geometry<double>& geometry )
		{
		      for( int i = 1 ; i <= element_index.size ; ++i )
		      {
			  int element_id = element_index.entry[i];
			  int number_neighbors = 0;
			  double weight_sum = 0;
			  // Get number of neighbors
			  for( int j = 1 ; j <= element_index.size ; ++j )
			  {
				int neighbor_element_id = element_index.entry[ j ];
				// Calculate distance between element and else elements
				double distance = 0;
				for( int dim = 1 ; dim <= geometry.nodes.dimension ; dim++ )
				{
				    double element_centroid  = elemental_centroids.entry[ element_id          ][ dim ];
				    double neighbor_centroid = elemental_centroids.entry[ neighbor_element_id ][ dim ];
				    double aux = element_centroid - neighbor_centroid ;
				    distance += ( aux * aux );
				}
				distance = sqrt( distance );
				// Check if element is in neighborhood
				if( distance <= ratio )
				{
				    // Add element to neighborhood list
				    Matrix<double> tmp( 2 , number_neighbors );
				    tmp = elemental_neighborhoods.entry[ element_id ];
				    number_neighbors ++;
				    elemental_neighborhoods.entry[ element_id ].Resize( 2, number_neighbors );
				    for ( int k = 1 ; k <= number_neighbors - 1 ; k++ )
				    {
					elemental_neighborhoods.entry[ element_id ].entry[ 1 ][ k ] = tmp.entry[ 1 ][ k ];
					elemental_neighborhoods.entry[ element_id ].entry[ 2 ][ k ] = tmp.entry[ 2 ][ k ];
				    }
				    // Add element id of neighbor
				    elemental_neighborhoods.entry[ element_id ].entry[ 1 ][ number_neighbors ] = (double)neighbor_element_id;
				    // Add element weight of neighbor
				    double weight = (double)ratio - (double)distance;
				    if( weight == 0.0 )
				    {
					// To avoid numeric errors
					elemental_neighborhoods.entry[ element_id ].entry[ 2 ][ number_neighbors ] = 1e-3;
					weight_sum += 1e-3;
				    }
				    else
				    {
					elemental_neighborhoods.entry[ element_id ].entry[ 2 ][ number_neighbors ] = weight;
					weight_sum += weight;
				    }
				}
			  }
			  // Normalize weights
			  for( int j = 1 ; j <= number_neighbors ; ++j )
			  {
			      elemental_neighborhoods.entry[ element_id ].entry[ 2 ][ j ] /= weight_sum;
			  }
		      }
		}
		
		
		
		void Sensitivities( Vector<int>& element_index )
		{
			compliance = 0;
			for( int i = 1 ; i <= element_index.size ; ++i)
			{
			    int element_id = element_index.entry[ i ];
			    double xe = design_variables.entry[ element_id ];
			    double uk = 0 , uku = 0;
			    Matrix<double> KeTMP( N , N );
			    Vector<double> ueTMP( N );
			    KeTMP = Ke.entry[ element_id ];
			    ueTMP = ue.entry[ element_id ];
			    for( int j = 1 ; j <= N ; ++j)
			    {
				    uk = 0;
				    for( int k = 1 ; k <= N ; ++k)
				    {
					uk += ueTMP.entry[ k ] * KeTMP.entry[ k ][ j ];
					if( k == N )
					{
					    uku += ( uk * ueTMP.entry[ j ] );
					}
				    }
			    }
			    double derivative = (double)p * ( pow( xe , (int)(p-1) ) ) * uku ;
			    sensitivities.entry[ element_id ] = derivative ;
			    
			    compliance += pow( xe , (int)(p) ) * uku ;
			}
			
		}
		
		void Filtering( Vector<int>& element_index )
		{
			maximal_filtering = 0;
			for( int i = 1 ; i <= element_index.size ; ++i )
			{
			    int element_id = element_index.entry[ i ];
			    double den = 0;
			    double num = 0;
			    for( int j = 1 ; j <= elemental_neighborhoods.entry[ element_id ].columns ; j++ )
			    {
				  int index = (int)elemental_neighborhoods.entry[ element_id ].entry[ 1 ][ j ];
				  double weight = elemental_neighborhoods.entry[ element_id ].entry[ 2 ][ j ];
				  num += ( design_variables.entry[ index ] * sensitivities.entry[ index ] * weight );
				  den += ( design_variables.entry[ element_id ] * weight);
			    }
			    filtering.entry[ element_id ] = num/den;
			    //Get max compliance derivate
			    if( filtering.entry[ element_id ] > maximal_filtering )
			    {
				maximal_filtering = filtering.entry[ element_id ];
			    }
			}
		}
		
		void Update_densities( Vector<int>& element_index )
		{
		 	lagrange = 0;
			double inf_limit = 0, sup_limit = maximal_filtering * 1e4;
			double xmin = thickness_empty_material , xmax = 1.0;
			Vector<double> 	Be(element_index.size);
			while(  sup_limit - inf_limit  > 1e-2 )
			{
			    lagrange = 0.5 * ( sup_limit + inf_limit );
			    volume = 0;

			    for( int i = 1 ; i <= element_index.size ; ++i )
			    {
				int element_id = element_index.entry[ i ];
				Be.entry[ element_id ] = filtering.entry[ element_id ] / ( lagrange * elemental_volumes.entry[ element_id ] / domain_volume );
				double xe = design_variables.entry[ element_id ];
				double xeBe = xe * sqrt( Be.entry[ element_id ] );
				
				double max_move = ( xmin > xe - mov ) ? xmin : ( xe - mov );
				double min_move = ( xmax < xe + mov ) ? xmax : ( xe + mov );
				
				if( xeBe <= max_move )
				{
				    next_design_variables.entry[ element_id ] = max_move;
				}
				else if( max_move < xeBe && xeBe < min_move )
				{
				    next_design_variables.entry[ element_id ] = xeBe;
				}
				else if( min_move <= xeBe )
				{
				    next_design_variables.entry[ element_id ] = min_move;
				}
				volume += ( next_design_variables.entry[element_id] * elemental_volumes.entry[element_id] );
			    }
			    // update lagrange multiplier range
			    if( (volume/domain_volume)  > volume_constraint )
			    {
				inf_limit = lagrange;
			    }
			    else						
			    {
				sup_limit = lagrange;
			    }
			} 
			// Objective function
			objective_function = compliance + ( lagrange *  fabs( ( volume / domain_volume ) - volume_constraint ) ) ;
		}
		
		bool Design_variables_stopping_criterion( Vector<int>& element_index )
		{
		  	bool stop;
			if( iteration < maximal_iterations )
			{
			  
			    design_variables_convergence = 0;
			    for(int i = 1 ; i < element_index.size ; i++ )
			    {
				  int element_id = element_index.entry[i];
				  // Diference Norm ,  x = f( x ) , || x - f( x ) ||
				  design_variables_convergence += ( ( next_design_variables.entry[ element_id ] - design_variables.entry[ element_id ] ) *
							( next_design_variables.entry[ element_id ] - design_variables.entry[ element_id ] ) );
			    }
			    design_variables_convergence = sqrt( design_variables_convergence );
			    if( design_variables_convergence < eps_convergence )
			    {
				stop = true;
			    }
			    else
			    {
				stop = false;
			    }
			}
			else
			{
			      stop = true;
			}
			
			design_variables = next_design_variables;
			return stop;
		}

		
		bool Sf_variance_stopping_criterion( )
		{
		  	bool stop_criterion1, stop_criterion2;
			if( iteration < maximal_iterations )
			{
				// Security factor criterion
				if( iteration != 1 )
				{
				    security_factor_convergence = fabs( ( maximal_vonmises / YS ) - ( last_maximal_vonmises / YS ) );
				    last_maximal_vonmises = maximal_vonmises ;
				    if(  security_factor_convergence < eps_convergence )
				    {
					stop_criterion1 = true ;
				    }
				    else
				    {
					stop_criterion1 = false ;
				    }
				}
				else 
				{
				    last_maximal_vonmises = maximal_vonmises;
				    stop_criterion1 = false ;
				}
				// Variance criterion
				if( iteration <= variance_convergence_parameter )
				{
				    historical_variance.entry[ iteration ] = objective_function;
				    stop_criterion2 = false ;
				}
				else
				{
				    // Get variance
				    double mean = 0 , var = 0;
				    for( int i = 1 ; i <= variance_convergence_parameter ; i++)
				    {
					mean += ( historical_variance.entry[ i ] );
				    }
				    mean /= (double)variance_convergence_parameter;
				    for( int i = 1 ; i <= variance_convergence_parameter ; i++)
				    {
					var += pow( historical_variance.entry[ i ] - mean , 2 );
				    }
				    historical_variance_convergence = var / (double)variance_convergence_parameter;
				    // Update variance
				    for( int i = 1 ; i <= variance_convergence_parameter - 1 ; i++)
				    {
					historical_variance.entry[ i ] = historical_variance.entry[ i + 1 ];
				    }
				    historical_variance.entry[ variance_convergence_parameter ] = objective_function;
				    // return variance
				    if( historical_variance_convergence < eps_convergence )
				    {
					stop_criterion2 = true ;
				    }
				    else
				    {
					stop_criterion2 = false ;
				    }
				}
			}
			else
			{
			      stop_criterion1 = true;
			      stop_criterion2 = true;
			}
			
			design_variables = next_design_variables;
			return ( stop_criterion1 & stop_criterion2 );
		}
		
		bool volume_stopping_criterion()
		{
			bool stop;
			if( iteration < maximal_global_iterations )
			{
			      volume_convergence = fabs( last_volume_constraint - volume_constraint );
			      double residue = security_factor_convergence - security_factor;
			      bool criterion1 = ( volume_convergence  < eps_convergence &&  security_factor < security_factor_convergence  ); 
			      bool criterion2 = ( residue > 0.0 && residue < eps_convergence );
			      if( criterion1 || criterion2 ) 
			      {
				  stop = true;
			      }
			      else
			      {
				  stop = false;
			      }
			}
			else
			{
			      stop =  true;
			}
			return stop;
		}
		
		void Get_displacements_data( Solid<double>& solid,
					     Geometry<double>& geometry,
					     const char* displacements_file_name
					  )
		{
			
		        FILE *displacements_report = fopen( displacements_file_name , "w" );
			// Number of forced conditioned nodes
			int nforced_nodes = solid.nodal_forces.size;
			displacement_avg = 0;
			
			// Initialize avg
			for ( int dim = 1 ; dim <= geometry.nodes.dimension ; dim++ )
			{
			    displacement_avg_coord.entry[ dim ] = 0.0 ;
			}
			fprintf( displacements_report , "Node_ID disp_X displ_X^2 disp_Y disp_Y^2 (disp_X^2+disp_Y^2) |Disp_X_Y| Acum(|Disp_X_Y|)\n" );
			// Calculate displacement avg per dimension of conditioned coordinates
			for ( int i = 1 ; i <= nforced_nodes ; i++ )
			{
			    // Get id of forced node
			    int forced_node_id = solid.nodal_forces.entry[ i ].node_id;
			    fprintf( displacements_report, "%d ",forced_node_id );
			    // Initialize norm of node
			    double norm_displacement_node = 0 ;
			    // Sum displacement per dimension 
			    for ( int dim = 1 ; dim <= geometry.nodes.dimension ; dim++ )
			    {
				double displacement = u.entry[ ( ( forced_node_id - 1 ) * geometry.nodes.dimension ) + dim ];
				fprintf( displacements_report, " %e ", displacement );
				// Avg per dimension
				displacement_avg_coord.entry[ ((dim - 1)*2)  + 1] += ( displacement );
				displacement_avg_coord.entry[ ((dim - 1)*2)  + 2] += fabs( displacement );
				// Calculating norm of node
				norm_displacement_node += ( displacement * displacement );
				fprintf( displacements_report, " %e ", displacement * displacement  );
			    }
			    //Get norm of node and sum for avg
			    fprintf( displacements_report, " %e ", norm_displacement_node  );
			    norm_displacement_node = sqrt(norm_displacement_node);
			    displacement_avg += norm_displacement_node;
			    fprintf( displacements_report, " %e  %e\n", norm_displacement_node ,displacement_avg);
			}
			// Get avg
			fprintf( displacements_report , "Avg(disp_X) Avg(abs(disp_X)) Avg(disp_Y) Avg(abs(disp_Y)) Avg(|Disp_X_Y|)\n" );
			displacement_avg /=(double)nforced_nodes;
			for ( int dim = 1 ; dim <= geometry.nodes.dimension * 2 ; dim++ )
			{
			    displacement_avg_coord.entry[ dim ] /= (double)nforced_nodes ;
			    fprintf( displacements_report, " %e ", displacement_avg_coord.entry[ dim ] );
			}
			fprintf( displacements_report, " %e ", displacement_avg );
			fclose(displacements_report);
		}
		
		void Get_efficiency_data( Vector<int>& element_index )
		{
			stress_avg = 0;
			stress_sd = 0;
			int element_counter = 0;
			for( int i = 1 ; i<= element_index.size ; i++ )
			{
			      if( design_variables.entry[ i ] > 0.5 )
			      {
				  stress_avg += ( elemental_vonmises.entry[ i ] / YS ) ;
				  element_counter++;
			      }
			}
			stress_avg/= element_counter;
			for( int i = 1 ; i<= element_index.size ; i++ )
			{
			      if( design_variables.entry[ i ] > 0.5 )
			      {
				  double aux = ( elemental_vonmises.entry[ i ] / YS ) - stress_avg ;
				  stress_sd += ( aux * aux ) ;
			      }
			}
			stress_sd = sqrt( stress_sd / element_counter ) ;
		}
		
		void Report_iteration_results( File& results_file , FILE* GID_file , Vector<int>& element_index , Vector<int>& node_index , Solid<double>& solid )
		{
			if( solid.save_displacement )		solid.WriteDisplacements(results_file, node_index , material_nodes , u , minimal_displacements_per_dimension , iteration );
			if( solid.save_material )		solid.WriteMaterial(results_file, element_index, design_variables, iteration);
			if( solid.save_von_mises )
			{
			      solid.WriteVonMises(results_file, element_index, design_variables, elemental_vonmises, iteration);
			      solid.WriteStress(results_file, element_index, design_variables, elemental_vonmises_per_dimension, iteration);  
			}
			if( solid.save_sensitivities )		solid.WriteSensitivities(results_file, element_index, design_variables, iteration);
			if( solid.save_filtering )		solid.WriteFiltering(results_file, element_index, design_variables, iteration);
			if( iteration == 1 )
			{
			      fprintf(GID_file , "iteration\tobjective_function\tvolume\tsecurity_factor\n");
			}
			fprintf(GID_file , "%d\t",iteration);
			fprintf(GID_file , "%f\t",objective_function);
			fprintf(GID_file , "%f\t",volume);
			fprintf(GID_file , "%f\n",security_factor);
		}
		
		void Report_final_results( File& results_file , FILE* GID_file , Vector<int>& element_index , Vector<int>& node_index , Solid<double>& solid )
		{
			if( solid.save_displacement )		solid.WriteDisplacements(results_file, node_index , material_nodes , u , minimal_displacements_per_dimension , iteration );
			if( solid.save_material )		solid.WriteMaterial(results_file, element_index, design_variables, iteration);
			if( solid.save_von_mises )
			{
			      solid.WriteVonMises(results_file, element_index, design_variables, elemental_vonmises, iteration);
			      solid.WriteStress(results_file, element_index, design_variables, elemental_vonmises_per_dimension, iteration);  
			}
			if( solid.save_sensitivities )		solid.WriteSensitivities(results_file, element_index, design_variables, iteration);
			if( solid.save_filtering )		solid.WriteFiltering(results_file, element_index, design_variables, iteration);
			if( optimization_method == 1 )
			{
			    printf( "- Method : SIMP\n" );	 fprintf(GID_file,"- Method : SIMP\n" );
			}
			else if( optimization_method == 2 )
			{
			    printf( "- Method : SIMP-SVC\n" );	 fprintf(GID_file,"- Method : SIMP-SVC\n" );
			}
			else if( optimization_method == 3 )
			{
			    printf( "- Method : SIMP-SC\n" );	 fprintf(GID_file,"- Method : SIMP-SC\n" );
			}
			printf( "- Iterations :   %d\n", iteration);					fprintf( GID_file , "- Iterations :   %d\n", iteration);					
			printf( "- Initial Volume :   %e\n", domain_volume );				fprintf( GID_file , "- Initial Volume :   %e\n", domain_volume );				
			printf( "- Final Volume :   %f\n", volume);					fprintf( GID_file , "- Final Volume :   %f\n", volume);					
			printf( "- Final Fraction Volume :   %e\n", volume/domain_volume );		fprintf( GID_file , "- Final Fraction Volume :   %e\n", volume/domain_volume );		
			printf( "- F( x ) :  %e\n", objective_function );				fprintf( GID_file , "- F( x ) :  %e\n", objective_function );				
			printf( "- Xdispl mean : %e \n", displacement_avg_coord.entry[ 1 ] );		fprintf( GID_file , "- Xdispl mean : %e \n", displacement_avg_coord.entry[ 1 ] );		
			printf( "- Xdispl abs mean : %e \n", displacement_avg_coord.entry[ 2 ] );	fprintf( GID_file , "- Xdispl abs mean : %e \n", displacement_avg_coord.entry[ 2 ] );	
			printf( "- Ydispl mean : %e \n", displacement_avg_coord.entry[ 3 ] );		fprintf( GID_file , "- Ydispl mean : %e \n", displacement_avg_coord.entry[ 3 ] );		
			printf( "- Ydispl abs mean : %e \n", displacement_avg_coord.entry[ 4 ] );	fprintf( GID_file , "- Ydispl abs mean : %e \n", displacement_avg_coord.entry[ 4 ] );	
			printf( "- |Displ | : %e \n", displacement_avg );				fprintf( GID_file , "- |Displ | : %e \n", displacement_avg );				
			printf( "- SF : %e \n", security_factor );					fprintf( GID_file , "- SF : %e \n", security_factor );					
			printf( "- Max VM : %e \n", maximal_vonmises );					fprintf( GID_file , "- Max VM : %e \n", maximal_vonmises );					
			printf( "- YS : %e \n", YS );							fprintf( GID_file , "- YS : %e \n", YS );							
			printf( "- Avg(Eff) : %e \n", stress_avg );					fprintf( GID_file , "- Avg(Eff) : %e \n", stress_avg );					
			printf( "- SD(Eff) : %e \n", stress_sd );					fprintf( GID_file , "- SD(Eff) : %e \n", stress_sd );					
			printf( "- Time(secs) : %f \n", difftime(time(NULL),start_time));		fprintf( GID_file , "- Time(secs) : %f \n", difftime(time(NULL),start_time));		
		}
};



#endif