#include "Solid.h"
#include "TopologyOptimizationAlgorithm.h"

#include <Basic/Log.h>
#include <Basic/Macros.h>
#include <Container/CSRMatrix.h>
#include <Container/Vector.h>
#include <FiniteElement/Assembler.h>
#include <FiniteElement/Geometry.h>
#include <FiniteElement/ShapeFunctions.h>
#include <Math/Distribution.h>
#include <Solver/SolverParameters.h>

#define PREFIX_MAXIMUM_LENGTH 900
#define PATH_MAXIMUM_LENGTH 1000

#include <stdio.h>


int main(int argc, char** argv)
{
	char geometry_file_name[PATH_MAXIMUM_LENGTH];
	char problem_file_name[PATH_MAXIMUM_LENGTH];
	char solver_file_name[PATH_MAXIMUM_LENGTH];
	char optimization_file_name[PATH_MAXIMUM_LENGTH];
	char results_file_name[PATH_MAXIMUM_LENGTH];
	char GID_file_name[PATH_MAXIMUM_LENGTH];
	char displacements_file_name[PATH_MAXIMUM_LENGTH];

	sprintf(solver_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.solver.dat", argv[1]);
	sprintf(optimization_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.optimization.dat", argv[1]);
	sprintf(geometry_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.geometry.dat", argv[1]);
	sprintf(problem_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.problem.dat", argv[1]);
	sprintf(results_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.post.res", argv[1]);
	sprintf(GID_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.GID.dat", argv[1]);
	sprintf(displacements_file_name, "%." MacroValueToString(PREFIX_MAXIMUM_LENGTH) "s.displacements.dat", argv[1]);

	SolverParameters<double> solver_parameters(solver_file_name);
	Geometry<double> geometry(geometry_file_name);
	Vector<int> element_index(geometry.mesh.elements_count);
	  element_index.FillSeries(1, 1);
	Vector<int> node_index(geometry.mesh.nodes_count);
	  geometry.mesh.GetElementsNodes(element_index, solver_parameters.reorder, node_index);
	ShapeFunctions<double> shape_functions(geometry.mesh, geometry.nodes);
	Solid<double> solid(problem_file_name, shape_functions, geometry.material_index);
	Assembler assembler(geometry.mesh, element_index, node_index, geometry.nodes.dimension);
	
	// Results 
	FILE* GID_file = fopen( GID_file_name , "w" );
	File results_file;
	results_file.Create(results_file_name);
	results_file.Write("GiD Post Result File 1.0\n\n");
	solid.WriteMaterialGaussPoints(results_file);
	
	// Load optimization alghorithm parameters
	SIMP<double> method( optimization_file_name , geometry,node_index,element_index,solid,assembler);
	int optimization_method = method.optimization_method;

	if( optimization_method == 1 || optimization_method == 2 ) //SIMP - SIMPSVC
	{
	      bool stopping;
	      do
	      {
		      method.FEM( element_index , node_index , solver_parameters , solid , shape_functions , geometry , assembler );
		      method.Sensitivities( element_index );
		      method.Filtering( element_index );
		      method.Update_densities( element_index );
		      method.Report_iteration_results( results_file , GID_file , element_index , node_index , solid );
		      if( optimization_method == 1 )
			  stopping = !method.Design_variables_stopping_criterion( element_index );
		      if( optimization_method == 2 )
			  stopping = !method.Sf_variance_stopping_criterion( );
		      method.iteration++;
	      }while( stopping );
	}
	else
	{
	      method.FEM( element_index , node_index , solver_parameters , solid , shape_functions , geometry , assembler );
	      if( method.security_factor > method.security_factor_convergence )
	      {
		  printf( " Loads too high. No structure with this domain capable to fulfill those loads\n - SF for full structure : %e\n",method.security_factor);
	      }
	      else
	      {
		    method.last_volume_constraint = 1.0;
		    method.volume_constraint = 0.5;
		    do{
			  method.iteration = 1;
			  method.last_maximal_vonmises = 5.0 * method.YS;
			  bool stopping;
			  do
			  {
				method.FEM( element_index , node_index , solver_parameters , solid , shape_functions , geometry , assembler );
				method.Sensitivities( element_index );
				method.Filtering( element_index );
				method.Update_densities( element_index );
				method.Report_iteration_results( results_file , GID_file , element_index , node_index , solid );
				stopping = !method.Sf_variance_stopping_criterion( );
				method.iteration++;
				method.global_iteration++;

			  }while( stopping );
			  
			  double volume_update = ( fabs ( method.last_volume_constraint - method.volume_constraint ) / 2.0 );
			  method.last_volume_constraint = method.volume_constraint;
			  if( method.security_factor >= method.sf_constraint )			//MVM bigger than  YS(over stress), increase volume to reduce stress
			  {
			      method.volume_constraint += volume_update;
			  }
			  else									//MVM lower than  YS(under stress), reduce volume to increase stress
			  {
			      method.volume_constraint -= volume_update;
			  }
		    }while( !method.volume_stopping_criterion() );
	      }
	      
	}
	
	method.Get_displacements_data( solid , geometry , displacements_file_name );
	method.Get_efficiency_data( element_index );
	
	method.Report_final_results( results_file , GID_file , element_index , node_index , solid );
	fclose( GID_file );
	return 0;
}
